<?php
/* --------------------------------------------------------------
   resource.get_usermod.php 2016-11-08
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2016 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class Smarty_Resource_Get_Usermod
 *
 * Resource overload for the smarty include statements. It enables USERMOD-files for templates.
 *
 * USAGE:
 * {include file="get_usermod:PATH/TO/TPL.FILE"}
 */
class Smarty_Resource_Get_Usermod extends Smarty_Resource_Custom
{
	
	/**
	 * Fetch a template and its modification time
	 *
	 * @param string  $name   template name
	 * @param string  $source template source
	 * @param integer $mtime  template modification timestamp (epoch)
	 *
	 * @return void
	 */
	protected function fetch($name, &$source, &$mtime)
	{
		$filename = get_usermod($this->_getPath($name));
		$source   = file_get_contents($filename);
		$mtime    = filemtime($filename);
	}
	
	
	/**
	 * Fetch a template's modification time
	 *
	 * @note implementing this method is optional. Only implement it if modification times can be accessed faster than
	 *       loading the complete template source.
	 *
	 * @param string $name template name
	 *
	 * @return integer timestamp (epoch) the template was modified
	 */
	protected function fetchTimestamp($name)
	{
		$filename = get_usermod($this->_getPath($name));
		
		return filemtime($filename);
	}
	
	
	/**
	 * @param $name
	 *
	 * @return string
	 */
	protected function _getPath($name)
	{
		if(defined('APPLICATION_RUN_MODE') && APPLICATION_RUN_MODE === 'backend')
		{
			return DIR_FS_ADMIN . 'html/content/' . $name;
		}
		
		return DIR_FS_CATALOG . $name;
	}
}